@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 - 2016 GraphicsMagick Group
+% Copyright (C) 2003 - 2019 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -53,7 +53,7 @@
   Typedef declarations.
 */
 typedef enum
-{ 
+{
   DisabledOutput = 0x0000,
   UndefinedOutput = 0x0000,
   StdoutOutput = 0x0001,
@@ -65,14 +65,11 @@ typedef enum
   MethodOutput = 0x0040
 } LogOutputType;
 
-typedef struct _OutputInfo
-{
-  const char *name;
-  LogOutputType mask;
-} OutputInfo;
-
 typedef struct _LogInfo
 {
+  SemaphoreInfo
+    *log_semaphore;
+
   FILE
     *file;
 
@@ -94,20 +91,15 @@ typedef struct _LogInfo
   LogMethod
     method;  /* Logger callback function */
 
+  MagickBool
+    log_configured;
+
   char
     path[256],
     filename[256],
     format[200];
 
 } LogInfo;
-
-typedef struct _EventInfo
-{
-  const char *name;
-  LogEventType mask;
-  int start_type;
-  int end_type;
-} EventInfo;
 
 /*
   This table maps between masks and the various event id's that can occur
@@ -126,63 +118,72 @@ typedef struct _EventInfo
      RegistryBase
 
  */
-static const EventInfo eventmask_map[] =
+static const struct
+{
+  const char name[14];
+  unsigned int name_len;
+  LogEventType mask;
+  int start_type;
+  int end_type;
+} eventmask_map[] =
   {
-    { "none", NoEventsMask, 0, 0 },
-    { "information", InformationEventMask, EventException, EventException+99 },
-    { "warning", WarningEventMask, WarningException, WarningException+99 },
-    { "error", ErrorEventMask, ErrorException, ErrorException+99 },
-    { "fatalerror", FatalErrorEventMask, FatalErrorException, FatalErrorException+99 },
-    { "configure", ConfigureEventMask, ConfigureBase, ConfigureBase },
-    { "annotate", AnnotateEventMask, AnnotateBase, AnnotateBase },
-    { "render", RenderEventMask, RenderBase, RenderBase },
-    { "transform", TransformEventMask, TransformBase, TransformBase },
-    { "locale", LocaleEventMask, LocaleBase, LocaleBase },
-    { "coder", CoderEventMask, CoderBase, CoderBase },
-    { "x11", X11EventMask, X11Base, UserBase },
-    { "cache", CacheEventMask, CacheBase, CacheBase },
-    { "blob", BlobEventMask, BlobBase, BlobBase },
-    { "deprecate", DeprecateEventMask, DeprecateBase, DeprecateBase },
-    { "user", UserEventMask, UserBase, UserBase },
-    { "resource", ResourceEventMask, ResourceBase, ResourceBase },
-    { "temporaryfile", TemporaryFileEventMask, TemporaryFileBase, TemporaryFileBase },
+#define STATIC_EVMASK(name,mask,start_type,end_type) {name,sizeof(name)-1,mask,start_type,end_type}
+    STATIC_EVMASK( "none", NoEventsMask, 0, 0 ),
+    STATIC_EVMASK( "information", InformationEventMask, EventException, EventException+99 ),
+    STATIC_EVMASK( "warning", WarningEventMask, WarningException, WarningException+99 ),
+    STATIC_EVMASK( "error", ErrorEventMask, ErrorException, ErrorException+99 ),
+    STATIC_EVMASK( "fatalerror", FatalErrorEventMask, FatalErrorException, FatalErrorException+99 ),
+    STATIC_EVMASK( "configure", ConfigureEventMask, ConfigureBase, ConfigureBase ),
+    STATIC_EVMASK( "annotate", AnnotateEventMask, AnnotateBase, AnnotateBase ),
+    STATIC_EVMASK( "render", RenderEventMask, RenderBase, RenderBase ),
+    STATIC_EVMASK( "transform", TransformEventMask, TransformBase, TransformBase ),
+    STATIC_EVMASK( "locale", LocaleEventMask, LocaleBase, LocaleBase ),
+    STATIC_EVMASK( "coder", CoderEventMask, CoderBase, CoderBase ),
+    STATIC_EVMASK( "x11", X11EventMask, X11Base, UserBase ),
+    STATIC_EVMASK( "cache", CacheEventMask, CacheBase, CacheBase ),
+    STATIC_EVMASK( "blob", BlobEventMask, BlobBase, BlobBase ),
+    STATIC_EVMASK( "deprecate", DeprecateEventMask, DeprecateBase, DeprecateBase ),
+    STATIC_EVMASK( "user", UserEventMask, UserBase, UserBase ),
+    STATIC_EVMASK( "resource", ResourceEventMask, ResourceBase, ResourceBase ),
+    STATIC_EVMASK( "temporaryfile", TemporaryFileEventMask, TemporaryFileBase, TemporaryFileBase ),
     /* this one is actually not used anymore */
-    { "exception", ExceptionEventMask, ExceptionBase, ExceptionBase },
-    { "option", OptionEventMask, OptionBase, OptionBase },
-    { "all", AllEventsMask, 0, 0 },
-    { 0, NoEventsMask, 0, 0 }
+    STATIC_EVMASK( "exception", ExceptionEventMask, ExceptionBase, ExceptionBase ),
+    STATIC_EVMASK( "option", OptionEventMask, OptionBase, OptionBase ),
+    STATIC_EVMASK( "all", AllEventsMask, 0, 0 )
   };
 
-static const OutputInfo output_map[] =
+static const struct
+{
+  const char name[14];
+  unsigned int name_len;
+  LogOutputType mask;
+} output_map[] =
   {
-    { "none", UndefinedOutput },
-    { "disabled", DisabledOutput },
-    { "stdout", StdoutOutput },
-    { "stderr", StderrOutput },
-    { "xmlfile", XMLFileOutput },
-    { "txtfile", TXTFileOutput },
-    { "win32debug", Win32DebugOutput },
-    { "win32eventlog", Win32EventlogOutput },
-    { 0, UndefinedOutput }
+#define STATIC_OMAP(name,mask) {name,sizeof(name)-1,mask}
+    STATIC_OMAP( "none", UndefinedOutput ),
+    STATIC_OMAP( "disabled", DisabledOutput ),
+    STATIC_OMAP( "stdout", StdoutOutput ),
+    STATIC_OMAP( "stderr", StderrOutput ),
+    STATIC_OMAP( "xmlfile", XMLFileOutput ),
+    STATIC_OMAP( "txtfile", TXTFileOutput ),
+    STATIC_OMAP( "win32debug", Win32DebugOutput ),
+    STATIC_OMAP( "win32eventlog", Win32EventlogOutput )
   };
 
 /*
   Static declarations.
 */
-static volatile unsigned int
-  log_configured = False;
-
 static LogInfo
-  log_info;
-
-static SemaphoreInfo
-  *log_semaphore = (SemaphoreInfo *) NULL;
+*log_info = (LogInfo *) NULL;
 
 /*
   Forward declarations.
 */
 static MagickPassFail
   ReadLogConfigureFile(const char *,const unsigned int,ExceptionInfo *);
+
+static LogEventType
+ParseEvents(const char *event_string) MAGICK_FUNC_PURE;
 
 /*
   Parse an event specification string and return the equivalent bits.
@@ -203,10 +204,10 @@ static LogEventType ParseEvents(const char *event_string)
       while ((*p != 0) && (isspace((int)(*p)) || *p == ','))
         p++;
 
-      for (i=0; eventmask_map[i].name != 0; i++)
+      for (i=0; i < ArraySize(eventmask_map); i++)
         {
           if (LocaleNCompare(p,eventmask_map[i].name,
-                             strlen(eventmask_map[i].name)) == 0)
+                             eventmask_map[i].name_len) == 0)
             {
               events = (LogEventType) ((unsigned int) events |
                                        (unsigned int) eventmask_map[i].mask);
@@ -217,6 +218,7 @@ static LogEventType ParseEvents(const char *event_string)
 
   return events;
 }
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -238,21 +240,17 @@ static LogEventType ParseEvents(const char *event_string)
 */
 MagickExport void DestroyLogInfo(void)
 {
-
-  LockSemaphoreInfo(log_semaphore);
-
-  if (log_info.file != (FILE *) NULL)
-    if ((log_info.file != stdout) && (log_info.file != stderr))
+  if (log_info->file != (FILE *) NULL)
+    if ((log_info->file != stdout) && (log_info->file != stderr))
       {
-        (void) fprintf(log_info.file,"</log>\n");
-        (void) fclose(log_info.file);
-        log_info.file=(FILE *) NULL;
+        if (log_info->output_type == XMLFileOutput)
+          (void) fprintf(log_info->file,"</log>\n");
+        (void) fclose(log_info->file);
+        log_info->file=(FILE *) NULL;
       }
-  log_configured=False;
-
-  UnlockSemaphoreInfo(log_semaphore);
-
-  DestroySemaphoreInfo(&log_semaphore);
+  log_info->log_configured=False;
+  DestroySemaphoreInfo(&log_info->log_semaphore);
+  MagickFreeMemory(log_info);
 }
 
 /*
@@ -284,40 +282,47 @@ InitializeLogInfo(void)
   const char
     *p;
 
-  /*
-    Initialize logging semaphore
-  */
-  assert(log_semaphore == (SemaphoreInfo *) NULL);
-  log_semaphore=AllocateSemaphoreInfo();
+  assert(log_info == (LogInfo *) NULL);
 
   /*
     Initialize LogInfo
   */
-  (void) memset(&log_info,0,sizeof(LogInfo));
+  log_info=MagickAllocateMemory(LogInfo *,sizeof(LogInfo));
+  if (log_info == (LogInfo *) NULL)
+    MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
+      UnableToAllocateLogInfo);
+
+  /*
+    Initialize logging semaphore
+  */
+  log_info->log_semaphore=AllocateSemaphoreInfo();
 
   /*
     Lock for access (to make Coverity happy)
   */
-  LockSemaphoreInfo(log_semaphore);
+#if defined(__COVERITY__)
+  LockSemaphoreInfo(log_info->log_semaphore);
+#endif /* defined(__COVERITY__) */
 
-  (void) strlcpy(log_info.path,"(default)",sizeof(log_info.path));
-  (void) strlcpy(log_info.filename,"Magick-%d.log",sizeof(log_info.filename));
-  log_info.generations=3;
-  log_info.limit=2000;
-  (void) strlcpy(log_info.format,"%t %r %u %p %m/%f/%l/%d:\n  %e",
-                 sizeof(log_info.format));
-  log_info.file=(FILE *) NULL;
-  log_info.generation=0;
-  log_info.count=0;
-  log_info.events=NoEventsMask;
-#if defined(MSWINDOWS)
-  log_info.output_type=Win32EventlogOutput;
-#else
-  log_info.output_type=StderrOutput;
-#endif
-  GetTimerInfo(&log_info.timer);
+  log_info->file=(FILE *) NULL;
+  GetTimerInfo(&log_info->timer);
+  log_info->generations=3;
+  log_info->limit=2000;
+  log_info->generation=0;
+  log_info->count=0;
+  log_info->events=NoEventsMask;
+  log_info->output_type=StderrOutput;
+  log_info->method=0;
+  log_info->log_configured=MagickFalse;
 
-  UnlockSemaphoreInfo(log_semaphore);
+  (void) strlcpy(log_info->path,"(default)",sizeof(log_info->path));
+  (void) strlcpy(log_info->filename,"Magick-%d.log",sizeof(log_info->filename));
+  (void) strlcpy(log_info->format,"%t %r %u %p %m/%f/%l/%d:\n  %e",
+                 sizeof(log_info->format));
+
+#if defined(__COVERITY__)
+  UnlockSemaphoreInfo(log_info->log_semaphore);
+#endif /* defined(__COVERITY__) */
 
   /*
     Set initial logging flags using the value of MAGICK_DEBUG if it is
@@ -383,7 +388,7 @@ InitializeLogInfoPost(void)
   /*
     Try to read the log configuration file.
   */
-  if (!log_configured)
+  if (!log_info->log_configured)
   {
     ExceptionInfo
       exception;
@@ -425,7 +430,7 @@ InitializeLogInfoPost(void)
 */
 MagickExport MagickBool IsEventLogging(void)
 {
-  return (log_info.events != NoEventsMask);
+  return (log_info->events != NoEventsMask);
 }
 
 /*
@@ -491,17 +496,17 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
   if (!IsEventLogging())
     return(False);
 
-  if (log_info.events != AllEventsMask)
+  if (log_info->events != AllEventsMask)
     {
       unsigned int
         i;
-      
+
       unsigned int
         enabled;
-      
+
       /* first translate the base type of the event to a mask */
       enabled=False;
-      for (i=0; eventmask_map[i].name != 0; i++)
+      for (i=0; i < ArraySize(eventmask_map); i++)
         {
           /* if the range in the table is above 100 it represents raw
              event id's. These entry types are to look for specific
@@ -512,7 +517,7 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
               if (((int) type >= eventmask_map[i].start_type) &&
                   ((int) type <= eventmask_map[i].end_type))
                 {
-                  if (((unsigned int) log_info.events) &
+                  if (((unsigned int) log_info->events) &
                       ((unsigned int) eventmask_map[i].mask))
                     {
                       enabled=True;
@@ -528,7 +533,7 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
               if ((((int) type % 100) >= eventmask_map[i].start_type) &&
                   (((int) type % 100) <= eventmask_map[i].end_type))
                 {
-                  if (((unsigned int) log_info.events) &
+                  if (((unsigned int) log_info->events) &
                       ((unsigned int) eventmask_map[i].mask))
                     {
                       enabled=True;
@@ -546,7 +551,7 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
      reason */
   GetPathComponent(module,TailPath,srcname);
 
-  LockSemaphoreInfo(log_semaphore);
+  LockSemaphoreInfo(log_info->log_semaphore);
   switch (((unsigned int) type) % 100)
   {
     case UndefinedException: domain=(char *) "Undefined"; break;
@@ -609,121 +614,121 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
 #endif
   seconds=time((time_t *) NULL);
   time_meridian=localtime(&seconds);
-  elapsed_time=GetElapsedTime(&log_info.timer);
-  user_time=GetUserTime(&log_info.timer);
-  (void) ContinueTimer((TimerInfo *) &log_info.timer);
+  elapsed_time=GetElapsedTime(&log_info->timer);
+  user_time=GetUserTime(&log_info->timer);
+  (void) ContinueTimer((TimerInfo *) &log_info->timer);
   FormatString(timestamp,"%04d%02d%02d%02d%02d%02d",time_meridian->tm_year+
     1900,time_meridian->tm_mon+1,time_meridian->tm_mday,
     time_meridian->tm_hour,time_meridian->tm_min,time_meridian->tm_sec);
-  if (((unsigned int) log_info.output_type) & XMLFileOutput)
+  if (((unsigned int) log_info->output_type) & XMLFileOutput)
     {
       /*
         Log to a file in the XML format.
       */
-      log_info.count++;
-      if (log_info.count >= log_info.limit)
+      log_info->count++;
+      if (log_info->count >= log_info->limit)
         {
-          (void) fprintf(log_info.file,"</log>\n");
-          (void) fclose(log_info.file);
-          log_info.file=(FILE *) NULL;
-          log_info.count=0;
+          (void) fprintf(log_info->file,"</log>\n");
+          (void) fclose(log_info->file);
+          log_info->file=(FILE *) NULL;
+          log_info->count=0;
         }
-      if (log_info.file == (FILE *) NULL)
+      if (log_info->file == (FILE *) NULL)
         {
           char
             filename[MaxTextExtent];
 
-          (void) MagickSceneFileName(filename,log_info.filename,".%lu",
-                                     MagickFalse,log_info.generation);
-          log_info.file=fopen(filename,"w");
-          if (log_info.file == (FILE *) NULL)
+          (void) MagickSceneFileName(filename,log_info->filename,".%lu",
+                                     MagickFalse,log_info->generation);
+          log_info->file=fopen(filename,"w");
+          if (log_info->file == (FILE *) NULL)
             {
-              UnlockSemaphoreInfo(log_semaphore);
+              UnlockSemaphoreInfo(log_info->log_semaphore);
               return(False);
             }
-          (void) fprintf(log_info.file,"<?xml version=\"1.0\"?>\n");
-          (void) fprintf(log_info.file,"<log>\n");
-          log_info.generation++;
-          if (log_info.generation >= log_info.generations)
-            log_info.generation=0;
+          (void) fprintf(log_info->file,"<?xml version=\"1.0\"?>\n");
+          (void) fprintf(log_info->file,"<log>\n");
+          log_info->generation++;
+          if (log_info->generation >= log_info->generations)
+            log_info->generation=0;
         }
-      (void) fprintf(log_info.file,"<record>\n");
-      (void) fprintf(log_info.file,"  <timestamp>%.1024s</timestamp>\n",
+      (void) fprintf(log_info->file,"<record>\n");
+      (void) fprintf(log_info->file,"  <timestamp>%.1024s</timestamp>\n",
         timestamp);
-      (void) fprintf(log_info.file,
-        "  <elapsed-time>%ld:%02ld</elapsed-time>\n",
-        (long) (elapsed_time/60.0),(long) ceil(fmod(elapsed_time,60.0)));
-      (void) fprintf(log_info.file,"  <user-time>%0.3f</user-time>\n",
+      (void) fprintf(log_info->file,
+        "  <elapsed-time>%ld:%-9.6f</elapsed-time>\n",
+        (long) (elapsed_time/60.0),fmod(elapsed_time,60.0));
+      (void) fprintf(log_info->file,"  <user-time>%0.3f</user-time>\n",
         user_time);
-      (void) fprintf(log_info.file,"  <pid>%ld</pid>\n",(long) getpid());
-      (void) fprintf(log_info.file,"  <module>%.1024s</module>\n",srcname);
-      (void) fprintf(log_info.file,"  <function>%.1024s</function>\n",
+      (void) fprintf(log_info->file,"  <pid>%ld</pid>\n",(long) getpid());
+      (void) fprintf(log_info->file,"  <module>%.1024s</module>\n",srcname);
+      (void) fprintf(log_info->file,"  <function>%.1024s</function>\n",
         function);
-      (void) fprintf(log_info.file,"  <line>%lu</line>\n",line);
-      (void) fprintf(log_info.file,"  <domain>%.1024s</domain>\n",domain);
-      (void) fprintf(log_info.file,"  <severity>%.1024s</severity>\n",severity);
-      (void) fprintf(log_info.file,"  <event>%.1024s</event>\n",event);
-      (void) fprintf(log_info.file,"</record>\n");
-      (void) fflush(log_info.file);
-      UnlockSemaphoreInfo(log_semaphore);
+      (void) fprintf(log_info->file,"  <line>%lu</line>\n",line);
+      (void) fprintf(log_info->file,"  <domain>%.1024s</domain>\n",domain);
+      (void) fprintf(log_info->file,"  <severity>%.1024s</severity>\n",severity);
+      (void) fprintf(log_info->file,"  <event>%.1024s</event>\n",event);
+      (void) fprintf(log_info->file,"</record>\n");
+      (void) fflush(log_info->file);
+      UnlockSemaphoreInfo(log_info->log_semaphore);
       return(True);
     }
-  if (((unsigned int) log_info.output_type) & TXTFileOutput)
+  if (((unsigned int) log_info->output_type) & TXTFileOutput)
     {
       /*
         Log to a file in the TXT format.
       */
-      log_info.count++;
-      if (log_info.count >= log_info.limit)
+      log_info->count++;
+      if (log_info->count >= log_info->limit)
         {
-          (void) fclose(log_info.file);
-          log_info.file=(FILE *) NULL;
-          log_info.count=0;
+          (void) fclose(log_info->file);
+          log_info->file=(FILE *) NULL;
+          log_info->count=0;
         }
-      if (log_info.file == (FILE *) NULL)
+      if (log_info->file == (FILE *) NULL)
         {
           char
             filename[MaxTextExtent];
 
-          (void) MagickSceneFileName(filename,log_info.filename,".%lu",
-                                     MagickFalse,log_info.generation);
-          log_info.file=fopen(filename,"w");
-          if (log_info.file == (FILE *) NULL)
+          (void) MagickSceneFileName(filename,log_info->filename,".%lu",
+                                     MagickFalse,log_info->generation);
+          log_info->file=fopen(filename,"w");
+          if (log_info->file == (FILE *) NULL)
             {
-              UnlockSemaphoreInfo(log_semaphore);
+              UnlockSemaphoreInfo(log_info->log_semaphore);
               return(False);
             }
-          log_info.generation++;
-          if (log_info.generation >= log_info.generations)
-            log_info.generation=0;
+          log_info->generation++;
+          if (log_info->generation >= log_info->generations)
+            log_info->generation=0;
         }
-        (void) fprintf(log_info.file,
-                       "%.1024s %ld:%02ld %0.3f %ld %.1024s %.1024s %lu"
+        (void) fprintf(log_info->file,
+                       "%.1024s %ld:%-9.6f %0.3f %ld %.1024s %.1024s %lu"
                        " %.1024s %.1024s %.1024s\n",
                        timestamp, (long) (elapsed_time/60.0),
-                       (long) ceil(fmod(elapsed_time,60.0)),
+                       fmod(elapsed_time,60.0),
                        user_time, (long) getpid(), srcname, function, line, domain,
                        severity, event);
-      (void) fflush(log_info.file);
-      UnlockSemaphoreInfo(log_semaphore);
+      (void) fflush(log_info->file);
+      UnlockSemaphoreInfo(log_info->log_semaphore);
       return(True);
     }
 #if defined(MSWINDOWS)
-  if (log_info.output_type & Win32DebugOutput)
+  if (log_info->output_type & Win32DebugOutput)
     {
       char
         buffer[MaxTextExtent];
 
       FormatString(buffer,
-                   "%.1024s %ld:%02ld %0.3f %ld %.1024s %.1024s %lu %.1024s"
+                   "%.1024s %ld:%-9.6f %0.3f %ld %.1024s %.1024s %lu %.1024s"
                    " %.1024s %.1024s\n",
                    timestamp, (long) (elapsed_time/60.0),
-                   (long) ceil(fmod(elapsed_time,60.0)),
+                   fmod(elapsed_time,60.0),
                    user_time, (long) getpid(), srcname, function, line,
                    domain, severity, event);
       OutputDebugString(buffer);
     }
-  if (log_info.output_type & Win32EventlogOutput)
+  if (log_info->output_type & Win32EventlogOutput)
     {
 #define LOGGING_ERROR_CODE 0
       char
@@ -736,10 +741,10 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
         hSource;
 
       FormatString(buffer,
-                   "%.1024s %ld:%02ld %0.3f %ld %.1024s %.1024s %lu %.1024s"
+                   "%.1024s %ld:%-9.6f %0.3f %ld %.1024s %.1024s %lu %.1024s"
                    " %.1024s %.1024s\n",
                    timestamp, (long) (elapsed_time/60.0),
-                   (long) ceil(fmod(elapsed_time,60.0)),
+                   fmod(elapsed_time,60.0),
                    user_time, (long) getpid(), srcname, function, line,
                    domain, severity, event);
       hSource = RegisterEventSource(NULL, MagickPackageName);
@@ -751,8 +756,8 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
         }
     }
 #endif
-  if ((((unsigned int) log_info.output_type) & StdoutOutput) ||
-      (((unsigned int) log_info.output_type) & StderrOutput))
+  if ((((unsigned int) log_info->output_type) & StdoutOutput) ||
+      (((unsigned int) log_info->output_type) & StderrOutput))
     {
       FILE
         *file;
@@ -760,9 +765,9 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
         Log to stdout in a "human readable" format.
       */
       file = stdout;
-      if (((unsigned int) log_info.output_type) & StderrOutput)
+      if (((unsigned int) log_info->output_type) & StderrOutput)
         file = stderr;
-      for (p=log_info.format; *p != '\0'; p++)
+      for (p=log_info->format; *p != '\0'; p++)
       {
         /*
           Process formatting characters in text.
@@ -828,8 +833,8 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
           }
           case 'r':
           {
-            (void) fprintf(file,"%ld:%02ld",(long) (elapsed_time/60.0),
-              (long) ceil(fmod(elapsed_time,60.0)));
+            (void) fprintf(file,"%ld:%-9.6f",(long) (elapsed_time/60.0),
+              fmod(elapsed_time,60.0));
             break;
           }
           case 's':
@@ -859,22 +864,22 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
       (void) fprintf(file,"\n");
       (void) fflush(file);
     }
-  if ((log_info.output_type & MethodOutput) &&
-      (log_info.method != (LogMethod) NULL))
+  if ((log_info->output_type & MethodOutput) &&
+      (log_info->method != (LogMethod) NULL))
     {
       char
         buffer[MaxTextExtent];
 
       FormatString(buffer,
-                   "%.1024s %ld:%02ld %0.3f %ld %.1024s %.1024s %lu %.1024s"
+                   "%.1024s %ld:%-9.6f %0.3f %ld %.1024s %.1024s %lu %.1024s"
                    " %.1024s %.1024s\n",
                    timestamp, (long) (elapsed_time/60.0),
-                   (long) ceil(fmod(elapsed_time,60.0)),
+                   fmod(elapsed_time,60.0),
                    user_time, (long) getpid(), srcname, function, line,
                    domain, severity, event);
-      log_info.method(type,buffer);
+      log_info->method(type,buffer);
     }
-  UnlockSemaphoreInfo(log_semaphore);
+  UnlockSemaphoreInfo(log_info->log_semaphore);
   return(True);
 }
 MagickExport unsigned int LogMagickEvent(const ExceptionType type,
@@ -1023,7 +1028,7 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
       }
     if (LocaleCompare(keyword,"<magicklog>") == 0)
       {
-        (void) strlcpy(log_info.path,path,sizeof(log_info.path));
+        (void) strlcpy(log_info->path,path,sizeof(log_info->path));
         continue;
       }
     MagickGetToken(q,(char **) NULL,token,token_max_length);
@@ -1037,7 +1042,7 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
       case 'e':
       {
         if (LocaleCompare((char *) keyword,"events") == 0)
-          log_info.events = (LogEventType) ((unsigned int) log_info.events |
+          log_info->events = (LogEventType) ((unsigned int) log_info->events |
                                              (unsigned int) ParseEvents(token));
         break;
       }
@@ -1046,12 +1051,12 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
       {
         if (LocaleCompare((char *) keyword,"filename") == 0)
           {
-            (void) strlcpy(log_info.filename,token,sizeof(log_info.filename));
+            (void) strlcpy(log_info->filename,token,sizeof(log_info->filename));
             break;
           }
         if (LocaleCompare((char *) keyword,"format") == 0)
           {
-            (void) strlcpy(log_info.format,token,sizeof(log_info.format));
+            (void) strlcpy(log_info->format,token,sizeof(log_info->format));
             break;
           }
         break;
@@ -1061,7 +1066,7 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
       {
         if (LocaleCompare((char *) keyword,"generations") == 0)
           {
-            log_info.generations=MagickAtoL(token);
+            log_info->generations=MagickAtoL(token);
             break;
           }
         break;
@@ -1071,7 +1076,7 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
       {
         if (LocaleCompare((char *) keyword,"limit") == 0)
           {
-            log_info.limit=MagickAtoL(token);
+            log_info->limit=MagickAtoL(token);
             break;
           }
         break;
@@ -1081,12 +1086,12 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
       {
        if (LocaleCompare((char *) keyword,"output") == 0)
           {
-            int i;
+            unsigned int i;
 
-            for (i=0; output_map[i].name != 0; i++)
+            for (i=0; i < ArraySize(output_map); i++)
             {
               if (LocaleNCompare(token,output_map[i].name,
-                                 strlen(output_map[i].name)) == 0)
+                                 output_map[i].name_len) == 0)
                 {
                   /* We do not OR these flags despite the fact that
                      they are bit masks because they are still
@@ -1095,11 +1100,11 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
                      field and others to do their work, so they can
                      not be used together */
 
-                  LockSemaphoreInfo(log_semaphore);
+                  LockSemaphoreInfo(log_info->log_semaphore);
 
-                  log_info.output_type=output_map[i].mask;
+                  log_info->output_type=output_map[i].mask;
 
-                  UnlockSemaphoreInfo(log_semaphore);
+                  UnlockSemaphoreInfo(log_info->log_semaphore);
 
                   break;
                 }
@@ -1116,7 +1121,7 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
   MagickFreeMemory(xml);
 
   if ((depth == 0) && (status == MagickPass))
-    log_configured=True;
+    log_info->log_configured=True;
 
   return(status);
 }
@@ -1151,17 +1156,17 @@ MagickExport unsigned long SetLogEventMask(const char *events)
   LogEventType
     event_flags=NoEventsMask;
 
-  LockSemaphoreInfo(log_semaphore);
+  LockSemaphoreInfo(log_info->log_semaphore);
 
   if (events != NULL)
     {
       event_flags=ParseEvents(events);
-      log_info.events=event_flags;
+      log_info->events=event_flags;
     }
 
-  event_flags=log_info.events;
+  event_flags=log_info->events;
 
-  UnlockSemaphoreInfo(log_semaphore);
+  UnlockSemaphoreInfo(log_info->log_semaphore);
 
   (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
                         "Set log event mask: %s", events ? events : "None");
@@ -1195,11 +1200,11 @@ MagickExport unsigned long SetLogEventMask(const char *events)
 */
 MagickExport void SetLogFormat(const char *format)
 {
-  LockSemaphoreInfo(log_semaphore);
+  LockSemaphoreInfo(log_info->log_semaphore);
 
-  (void) strlcpy(log_info.format,format,sizeof(log_info.format));
+  (void) strlcpy(log_info->format,format,sizeof(log_info->format));
 
-  UnlockSemaphoreInfo(log_semaphore);
+  UnlockSemaphoreInfo(log_info->log_semaphore);
 }
 
 /*
@@ -1229,11 +1234,11 @@ MagickExport void SetLogFormat(const char *format)
 */
 MagickExport void SetLogMethod(LogMethod method)
 {
-  LockSemaphoreInfo(log_semaphore);
+  LockSemaphoreInfo(log_info->log_semaphore);
 
-  log_info.output_type=(LogOutputType) (log_info.output_type |
+  log_info->output_type=(LogOutputType) (log_info->output_type |
     MethodOutput);
-  log_info.method=method;
+  log_info->method=method;
 
-  UnlockSemaphoreInfo(log_semaphore);
+  UnlockSemaphoreInfo(log_info->log_semaphore);
 }
