@@ -1,12 +1,12 @@
 /*
-  Copyright (C) 2003 - 2013 GraphicsMagick Group
+  Copyright (C) 2003 - 2016 GraphicsMagick Group
   Copyright (C) 2002 ImageMagick Studio
- 
+
   This program is covered by multiple licenses, which are described in
   Copyright.txt. You should have received a copy of Copyright.txt with this
   package; otherwise see http://www.graphicsmagick.org/www/Copyright.html.
- 
-  Windows NT Utility Methods for ImageMagick.
+
+  Windows NT Utility Methods for GraphicsMagick.
 */
 #ifndef _MAGICK_NTBASE_H
 #define _MAGICK_NTBASE_H
@@ -25,6 +25,7 @@ extern "C" {
 #include <io.h>
 #include <process.h>
 #include <errno.h>
+#include <malloc.h>
 #if defined(_DEBUG)
 #include <crtdbg.h>
 #endif
@@ -38,11 +39,33 @@ extern "C" {
 #define W_OK 2
 #define RW_OK 6
 #define HAVE_VSNPRINTF 1
-#define HAVE_TEMPNAM 1
 #define HAVE_RAISE 1
 #define HAVE_SPAWNVP 1
 #define HAVE_PROCESS_H 1
 
+#if defined(_VISUALC_)
+#  define HAVE_CRYPTGENRANDOM 1
+#  define HAVE_WINCRYPT_H 1
+#endif
+
+/*
+  POSIX definitions for standard file numbers for use with _read() or _write()
+*/
+#if !defined(STDIN_FILENO)
+#  define STDIN_FILENO 0
+#endif
+#if !defined(STDOUT_FILENO)
+#  define STDOUT_FILENO 1
+#endif
+#if !defined(STDERR_FILENO)
+#  define STDERR_FILENO 2
+#endif
+
+/*
+  Size type passed to read/write
+*/
+#define MAGICK_POSIX_IO_SIZE_T unsigned int
+
 /*
   libtiff features.
 */
@@ -50,6 +73,9 @@ extern "C" {
 /* Define to 1 if you have the <tiffconf.h> header file. */
 #define HAVE_TIFFCONF_H 1
 
+/* Define to 1 if you have the `TIFFIsCODECConfigured' function. */
+#define HAVE_TIFFISCODECCONFIGURED 1
+
 /* Define to 1 if you have the `TIFFMergeFieldInfo' function. */
 #define HAVE_TIFFMERGEFIELDINFO 1
 
@@ -90,12 +116,34 @@ extern "C" {
 # endif
 #endif
 
-#if !defined(tempnam)
-# define tempnam _tempnam
+#if !defined(hypot)
+#  define hypot  _hypot
 #endif
 
+// We do not yet know what CRT versions have these float functions or if
+// special-handling (e.g. mapping to underscore version) is needed.
+// It is known that Visual Studio 2003 does not offer these functions.
+#if (defined(_MSC_VER) && _MSC_VER >= 1500)
+
+/* Define to 1 if you have the `cosf' function. */
+#define HAVE_COSF 1
+
+/* Define to 1 if you have the `fabsf' function. */
+#define HAVE_FABSF 1
+
+/* Define to 1 if you have the `logf' function. */
+#define HAVE_LOGF 1
+
+/* Define to 1 if you have the `sinf' function. */
+#define HAVE_SINF 1
+
+/* Define to 1 if you have the `sqrtf' function. */
+#define HAVE_SQRTF 1
+
+#endif // defined(_VISUALC_)
+
 #if !defined(_MSC_VER) || (defined(_MSC_VER) && _MSC_VER < 1500)
-#define vsnprintf _vsnprintf 
+#define vsnprintf _vsnprintf
 #endif
 
 #if defined(_MT) && defined(MSWINDOWS)
@@ -123,19 +171,35 @@ extern "C" {
   Visual C++ 7.0 supports GlobalMemoryStatusEx but MinGW headers and
   Visual C++ 6.0 lack support for it.
 
-  _MSC_VER values:
+  _MSC_VER values (see https://en.wikipedia.org/wiki/Microsoft_Visual_C%2B%2B):
     1100 MSVC 5.0
     1200 MSVC 6.0
     1300 MSVC 7.0 Visual C++ .NET 2002
-    1310 Visual c++ .NET 2003
+    1310 Visual C++ .NET 2003
     1400 Visual C++ 2005
     1500 Visual C++ 2008
+    1600 Visual C++ 2010
+    1700 Visual C++ 2012
+    1800 Visual C++ 2013
+    1900 Visual C++ 2015
+    1910 Visual C++ 2017
+    1920 Visual C++ 2019
+
+  Should look at __CLR_VER ("Defines the version of the common language
+  runtime used when the application was compiled.") as well.
 */
 #if defined(_VISUALC_) && (_MSC_VER >= 1310)
 #  define HAVE_GLOBALMEMORYSTATUSEX 1
 #endif
 
-#if defined(_VISUALC_) && (_MSC_VER >= 1300)
+/*
+  _aligned_malloc was introduced in Visual Studio .NET 2003.
+  The feature is is introduced in msvcr71.dll and is capable of
+  working on Windows '98 given that the correct run-time libraries
+  are installed.
+*/
+#if ((defined(_VISUALC_) && (_MSC_VER >= 1310)) ||              \
+     (defined(__MINGW32__) && (__MSVCRT_VERSION__ >= 0x0700)))
 #  define HAVE__ALIGNED_MALLOC 1
 #endif
 
@@ -164,6 +228,67 @@ extern "C" {
 #  define fileno(x) _fileno(x)
 #endif
 
+#if !defined(unlink)
+#  define unlink(path) _unlink(path)
+#endif /* !defined(unlink) */
+
+
+/*
+  I/O defines.
+*/
+#if !defined(Windows95) && !defined(__BORLANDC__)
+  /* Windows '95 and Borland C do not support _lseeki64 */
+#  define MagickSeek(fildes,offset,whence)  _lseeki64(fildes,/* __int64 */ offset,whence)
+#  define MagickTell(fildes) /* __int64 */ _telli64(fildes)
+#else
+#  define MagickSeek(fildes,offset,whence)  lseek(fildes,offset,whence)
+#  define MagickTell(fildes) (MagickSeek(fildes,0,SEEK_CUR))
+#endif
+
+
+#if !defined(Windows95) && !defined(__BORLANDC__) && \
+  !(defined(_MSC_VER) && _MSC_VER < 1400) && \
+  !(defined(__MSVCRT_VERSION__) && __MSVCRT_VERSION__ < 0x800)
+
+  /*
+    Windows '95 and Borland C do not support _lseeki64
+    Visual Studio does not support _fseeki64 and _ftelli64 until the 2005 release.
+    Without these interfaces, files over 2GB in size are not supported for Windows.
+  */
+#  define MagickFseek(stream,offset,whence) _fseeki64(stream,/* __int64 */ offset,whence)
+#  define MagickFstat(fildes,stat_buff) _fstati64(fildes,/* struct _stati64 */ stat_buff)
+#  define MagickFtell(stream) /* __int64 */ _ftelli64(stream)
+#  define MagickStatStruct_t struct _stati64
+#  define MagickStat(path,stat_buff) _stati64(path,/* struct _stati64 */ stat_buff)
+#else
+#  define MagickFseek(stream,offset,whence) fseek(stream,offset,whence)
+#  define MagickFstat(fildes,stat_buff) fstat(fildes,stat_buff)
+#  define MagickFtell(stream) ftell(stream)
+#  define MagickStatStruct_t struct stat
+#  define MagickStat(path,stat_buff) stat(path,stat_buff)
+#endif
+
+/*
+  The isnan() function was implemented starting in Visual Studio 2003
+  The function prototype for isnan() and _isnanf() comes from <math.h>
+  and the prototype for _isnan() comes from <float.h>.  The _isnanf()
+  function is only available in _WIN64 builds.
+
+  Some useful documentation was found at
+  https://msdn.microsoft.com/en-us/library/tzthab44.aspx.
+*/
+#if (defined(_MSC_VER) && _MSC_VER < 1700 && !defined(isnan))
+#  if defined(_WIN64)
+#    define isnan(f) _isnanf(f)
+#  else
+#    include <float.h> /* Needed for _isnan */
+#    if !defined(isnan)
+#      define isnan(f) _isnan(f)
+#    endif
+#  endif
+#endif
+
+
 /*
   Typedef declarations.
 */
@@ -276,7 +401,6 @@ extern MagickExport void
   System functions.
 */
 extern MagickExport int
-  Exit(int),
   NTSystemComman(const char *);
 
 #if !defined(XS_VERSION)  /* Not in Perl extension */
@@ -305,6 +429,7 @@ extern MagickExport double
 #define MS_SYNC         0x1  // synchronous page sync
 
 extern MagickExport void
+  Exit(int) MAGICK_FUNC_NORETURN,
   *NTmmap(char *address,size_t length,int protection,int access,int file,
      magick_off_t offset);
 
@@ -313,6 +438,12 @@ extern MagickExport int
   NTmsync(void *addr, size_t len, int flags),
   NTmunmap(void *addr, size_t len);
 
+extern MagickExport int
+  MagickGetFileAttributes(const char *filename, MagickStatStruct_t *statbuf);
+
+extern MagickExport int
+  MagickSetFileAttributes(const char *filename, const MagickStatStruct_t *statbuf);
+
 #define MagickMmap(address,length,protection,access,file,offset) \
   NTmmap(address,length,protection,access,file,offset)
 #define MagickMsync(addr,len,flags) NTmsync(addr,len,flags)
@@ -344,7 +475,7 @@ extern MagickExport const char
 #define lt_dlsetsearchpath(path) NTdlsetsearchpath(path)
 #define lt_dlsym(handle,name) NTdlsym(handle,name)
 #endif /* !defined(HasLTDL) */
-  
+
 extern MagickExport unsigned char
   *NTResourceToBlob(const char *);
 
@@ -362,6 +493,9 @@ extern MagickExport void
 extern MagickExport long
   MagickGetMMUPageSize();
 
+extern MagickExport void
+  NTInitializeExceptionHandlers();
+
 #endif /* !XS_VERSION */
 
 #if defined(__cplusplus) || defined(c_plusplus)
